//--------------------------------------------------------------
// File     : stm32_ub_syscheck.c
// Datum    : 01.01.2015
// Version  : 1.2
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, RCC
// Funktion : Globale Testfunktionen
//
// Hinweis  : falls der Assert-Mode benutzt werden soll,
//            muss im File "cmsis_boot/stm32f4xx_conf.h"
//            der define "USE_FULL_ASSERT" auf 1 gesetzt werden
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_syscheck.h"


#if  SYSCHECK_USE_CLOCK==1
//--------------------------------------------------------------
// testet alle Systemclocks auf die richtige Frequenz
// und auf die richtige Quelle
// Return_wert :
//    SYSCHECK_OK   , System ist OK
//  != SYSCHECK_OK  , System hat einen Fehler
//--------------------------------------------------------------
SYSCHECK_t UB_SysCheck_Clock(void)
{
  SYSCHECK_t ret_wert=SYSCHECK_OK;
  RCC_ClocksTypeDef RCC_Clocks;
  uint8_t clk_source;
  uint32_t clk_frq;

  // Clock-Quelle auslesen
  clk_source=RCC_GetSYSCLKSource();
  if(clk_source!=SYSCHECK_CLKSOURCE) return(CLKSOURCE_ERR);

  // Clock-Frequenzen auslesen
  RCC_GetClocksFreq(&RCC_Clocks);
  clk_frq = RCC_Clocks.HCLK_Frequency;
  if(clk_frq!=SYSCHECK_HCLK_FRQ) return(HCLK_ERR);
  clk_frq = RCC_Clocks.PCLK1_Frequency;
  if(clk_frq!=SYSCHECK_PCLK1_FRQ) return(PCLK1_ERR);
  clk_frq = RCC_Clocks.PCLK2_Frequency;
  if(clk_frq!=SYSCHECK_PCLK2_FRQ) return(PCLK2_ERR);
  clk_frq = RCC_Clocks.SYSCLK_Frequency;
  if(clk_frq!=SYSCHECK_SYSCLK_FRQ) return(SYSCLK_ERR);

  return(ret_wert);
}
#endif


#if  SYSCHECK_USE_ASSERT==1
//--------------------------------------------------------------
// init vom Assert-Mode
// (alle Pointer initialisieren)
//--------------------------------------------------------------
void UB_SysCheck_Assert_Init(void)
{
  SYSCHECK_ASSERT.status=ASSERT_NO_ERROR;
  SYSCHECK_ASSERT.err_line=0;


  //--------------------------------------------------
  ADC_DBG  = (ADC_Common_TypeDef *)  ADC_BASE;
  ADC1_DBG = (ADC_TypeDef *)  ADC1_BASE;
  ADC2_DBG = (ADC_TypeDef *)  ADC2_BASE;
  ADC3_DBG = (ADC_TypeDef *)  ADC3_BASE;
  //--------------------------------------------------
  CAN1_DBG = (CAN_TypeDef *)  CAN1_BASE;
  CAN2_DBG = (CAN_TypeDef *)  CAN2_BASE;
  //--------------------------------------------------
  CRC_DBG = (CRC_TypeDef *)  CRC_BASE;
  //--------------------------------------------------
  CRYP_DBG = (CRYP_TypeDef *)  CRYP_BASE;
  //--------------------------------------------------
  DAC_DBG = (DAC_TypeDef *)  DAC_BASE;
  //--------------------------------------------------
  DBGMCU_DBG = (DBGMCU_TypeDef *)  DBGMCU_BASE;
  //--------------------------------------------------
  DCMI_DBG = (DCMI_TypeDef *)  DCMI_BASE;
  //--------------------------------------------------
  DMA1_DBG = (DMA_TypeDef *)  DMA1_BASE;
  DMA1_Stream0_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream0_BASE;
  DMA1_Stream1_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream1_BASE;
  DMA1_Stream2_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream2_BASE;
  DMA1_Stream3_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream3_BASE;
  DMA1_Stream4_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream4_BASE;
  DMA1_Stream5_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream5_BASE;
  DMA1_Stream6_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream6_BASE;
  DMA1_Stream7_DBG = (DMA_Stream_TypeDef *)  DMA1_Stream7_BASE;
  DMA2_DBG = (DMA_TypeDef *)  DMA2_BASE;
  DMA2_Stream0_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream0_BASE;
  DMA2_Stream1_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream1_BASE;
  DMA2_Stream2_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream2_BASE;
  DMA2_Stream3_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream3_BASE;
  DMA2_Stream4_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream4_BASE;
  DMA2_Stream5_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream5_BASE;
  DMA2_Stream6_DBG = (DMA_Stream_TypeDef *)  DMA2_Stream6_BASE;
  DMA2_Stream7_DBG = (DMA_Stream_TypeDef *) DMA2_Stream7_BASE;
  //--------------------------------------------------
  EXTI_DBG = (EXTI_TypeDef *)  EXTI_BASE;
  //--------------------------------------------------
  ETH_DBG = (ETH_TypeDef *)  ETH_BASE;
  //--------------------------------------------------
  FLASH_DBG = (FLASH_TypeDef *)  FLASH_R_BASE;
  //--------------------------------------------------
  FSMC_Bank1_DBG  = (FSMC_Bank1_TypeDef *)  FSMC_Bank1_R_BASE;
  FSMC_Bank1E_DBG = (FSMC_Bank1E_TypeDef *)  FSMC_Bank1E_R_BASE;
  FSMC_Bank2_DBG  = (FSMC_Bank2_TypeDef *)  FSMC_Bank2_R_BASE;
  FSMC_Bank3_DBG  = (FSMC_Bank3_TypeDef *)  FSMC_Bank3_R_BASE;
  FSMC_Bank4_DBG  = (FSMC_Bank4_TypeDef *)  FSMC_Bank4_R_BASE;
  //--------------------------------------------------
  GPIOA_DBG = (GPIO_TypeDef *)  GPIOA_BASE;
  GPIOB_DBG = (GPIO_TypeDef *)  GPIOB_BASE;
  GPIOC_DBG = (GPIO_TypeDef *)  GPIOC_BASE;
  GPIOD_DBG = (GPIO_TypeDef *)  GPIOD_BASE;
  GPIOE_DBG = (GPIO_TypeDef *)  GPIOE_BASE;
  GPIOF_DBG = (GPIO_TypeDef *)  GPIOF_BASE;
  GPIOG_DBG = (GPIO_TypeDef *)  GPIOG_BASE;
  GPIOH_DBG = (GPIO_TypeDef *)  GPIOH_BASE;
  GPIOI_DBG = (GPIO_TypeDef *)  GPIOI_BASE;
  //--------------------------------------------------
  HASH_DBG = (HASH_TypeDef *)  HASH_BASE;
  //--------------------------------------------------
  I2C1_DBG = (I2C_TypeDef *)  I2C1_BASE;
  I2C2_DBG = (I2C_TypeDef *)  I2C2_BASE;
  I2C3_DBG = (I2C_TypeDef *)  I2C3_BASE;
  //--------------------------------------------------
  IWDG_DBG = (IWDG_TypeDef *)  IWDG_BASE;
  //--------------------------------------------------
  PWR_DBG = (PWR_TypeDef *)  PWR_BASE;
  //--------------------------------------------------
  RCC_DBG = (RCC_TypeDef *)  RCC_BASE;
  //--------------------------------------------------
  RNG_DBG = (RNG_TypeDef *)  RNG_BASE;
  //--------------------------------------------------
  RTC_DBG = (RTC_TypeDef *)  RTC_BASE;
  //--------------------------------------------------
  SDIO_DBG = (SDIO_TypeDef *)  SDIO_BASE;
  //--------------------------------------------------
  SPI1_DBG = (SPI_TypeDef *)  SPI1_BASE;
  SPI2_DBG = (SPI_TypeDef *)  SPI2_BASE;
  SPI3_DBG = (SPI_TypeDef *)  SPI3_BASE;
  //--------------------------------------------------
  SYSCFG_DBG = (SYSCFG_TypeDef *)  SYSCFG_BASE;
  //--------------------------------------------------
  TIM1_DBG  = (TIM_TypeDef *)  TIM1_BASE;
  TIM2_DBG  = (TIM_TypeDef *)  TIM2_BASE;
  TIM3_DBG  = (TIM_TypeDef *)  TIM3_BASE;
  TIM4_DBG  = (TIM_TypeDef *)  TIM4_BASE;
  TIM5_DBG  = (TIM_TypeDef *)  TIM5_BASE;
  TIM6_DBG  = (TIM_TypeDef *)  TIM6_BASE;
  TIM7_DBG  = (TIM_TypeDef *)  TIM7_BASE;
  TIM8_DBG  = (TIM_TypeDef *)  TIM8_BASE;
  TIM9_DBG  = (TIM_TypeDef *)  TIM9_BASE;
  TIM10_DBG = (TIM_TypeDef *)  TIM10_BASE;
  TIM11_DBG = (TIM_TypeDef *)  TIM11_BASE;
  TIM12_DBG = (TIM_TypeDef *)  TIM12_BASE;
  TIM13_DBG = (TIM_TypeDef *)  TIM13_BASE;
  TIM14_DBG = (TIM_TypeDef *)  TIM14_BASE;
  //--------------------------------------------------
  USART1_DBG = (USART_TypeDef *)  USART1_BASE;
  USART2_DBG = (USART_TypeDef *)  USART2_BASE;
  USART3_DBG = (USART_TypeDef *)  USART3_BASE;
  UART4_DBG  = (USART_TypeDef *)  UART4_BASE;
  UART5_DBG  = (USART_TypeDef *)  UART5_BASE;
  USART6_DBG = (USART_TypeDef *)  USART6_BASE;
  //--------------------------------------------------
  WWDG_DBG = (WWDG_TypeDef *)  WWDG_BASE;
}
#endif



#if  SYSCHECK_USE_ASSERT==1
//--------------------------------------------------------------
// testet ob ein Assert Fehler aufgetreten ist
// Return_wert :
//    SYSCHECK_OK   , kein Fehler aufgetreten
//    ASSERT_ERR    , Assert Fehler aufgetreten
//                    Hinweise stehen in "SYSCHECK_ASSERT"
//                      SYSCHECK_ASSERT.err_line => Zeilennummer
//                      SYSCHECK_ASSERT.err_file => Filename
//--------------------------------------------------------------
SYSCHECK_t UB_SysCheck_Assert(void)
{
  SYSCHECK_t ret_wert=SYSCHECK_OK;

  if(SYSCHECK_ASSERT.status!=ASSERT_NO_ERROR) ret_wert=ASSERT_ERR;

  return(ret_wert);
}
#endif



#if  SYSCHECK_USE_ASSERT==1
//--------------------------------------------------------------
// Globale Assert-Funktion
// (wird bei einem Fehler whrend der Laufzeit aufgerufen)
//--------------------------------------------------------------
void assert_failed(uint8_t* file, uint32_t line)
{ 
  uint32_t pos=0;

  //-----------------------------------------
  // hier z.B. eine LED einschalten
  // damit der Fehler erkannt wird
  //-----------------------------------------

  //-----------------------------------------
  // Fehler in der Struktur speichern
  //-----------------------------------------
  if(SYSCHECK_ASSERT.status==ASSERT_NO_ERROR) {
    // es wird nur der erste Fehler gespeichert
    SYSCHECK_ASSERT.status=ASSERT_ERROR;
    SYSCHECK_ASSERT.err_line=line;
    SYSCHECK_ASSERT.err_file=file;

    // Stringende suchen
    while(*file!=0) {
      file++;
      pos++;
    }
    // letzten 50 Zeichen vom Filenamen bergeben
    if(pos>51) {
    	file-=50;
    }
    else {
    	file=0;
    }
    SYSCHECK_ASSERT.err_file=file;
  }
}
#else
  void assert_failed(uint8_t* file, uint32_t line)
  {
    // leere Funktion
  }
#endif


#if SYSCHECK_USE_MCO==1
//--------------------------------------------------------------
// test der eingestellten Clock-Frq
// das Clock-Signal wird am Pin MCO1 (PA8) ausgegeben
// source    : [MCO1_LSE, MCO1_HSE, MCO1_HSI, MCO1_PLLCLK]
// prescaler : [MCO_DIV1] bis [MCO_DIV5]
//
// am STM32F407 Disco : LSE=OFF
//                      HSE=8MHz
//                      HSI=16MHz
//                      PLLCLK=168MHz
//--------------------------------------------------------------
void UB_SysCheck_MCO1(MCO1_SOURCE source, MCO_PRESCALER prescaler)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  uint32_t p_value;

  // Clock Enable
  RCC_AHB1PeriphClockCmd(MCO1_GPIO_CLOCK, ENABLE);

  // Config PA8 (MCO1) als AF
  GPIO_InitStructure.GPIO_Pin = MCO1_GPIO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MCO1_GPIO_PORT, &GPIO_InitStructure);

  // Vorteiler auswhlen
  if(prescaler==MCO_DIV1) p_value=RCC_MCO1Div_1;
  if(prescaler==MCO_DIV2) p_value=RCC_MCO1Div_2;
  if(prescaler==MCO_DIV3) p_value=RCC_MCO1Div_3;
  if(prescaler==MCO_DIV4) p_value=RCC_MCO1Div_4;
  if(prescaler==MCO_DIV5) p_value=RCC_MCO1Div_5;

  // Clock an PA8 ausgeben
  if(source==MCO1_LSE) RCC_MCO1Config(RCC_MCO1Source_LSE, p_value);
  if(source==MCO1_HSE) RCC_MCO1Config(RCC_MCO1Source_HSE, p_value);
  if(source==MCO1_HSI) RCC_MCO1Config(RCC_MCO1Source_HSI, p_value);
  if(source==MCO1_PLLCLK) RCC_MCO1Config(RCC_MCO1Source_PLLCLK, p_value);

  GPIO_PinAFConfig(MCO1_GPIO_PORT, MCO1_GPIO_SOURCE, GPIO_AF_MCO);
}
#endif


#if SYSCHECK_USE_MCO==1
//--------------------------------------------------------------
// test der eingestellten Clock-Frq
// das Clock-Signal wird am Pin MCO2 (PC9) ausgegeben
// source    : [MCO2_SYSCLOCK, MCO2_PLLI2SCLK, MCO2_HSE, MCO2_PLLCLK]
// prescaler : [MCO_DIV1] bis [MCO_DIV5]
//
// am STM32F407 Disco : Sysclock=168MHz
//                      I2Sclock=Off
//                      HSE=8MHZ
//                      PLLCLK=168MHz
//--------------------------------------------------------------
void UB_SysCheck_MCO2(MCO2_SOURCE source, MCO_PRESCALER prescaler)
{
  GPIO_InitTypeDef  GPIO_InitStructure;
  uint32_t p_value;

  // Clock Enable
  RCC_AHB1PeriphClockCmd(MCO2_GPIO_CLOCK, ENABLE);

  // Config PC9 (MCO2) als AF
  GPIO_InitStructure.GPIO_Pin = MCO2_GPIO_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(MCO2_GPIO_PORT, &GPIO_InitStructure);

  // Vorteiler auswhlen
  if(prescaler==MCO_DIV1) p_value=RCC_MCO2Div_1;
  if(prescaler==MCO_DIV2) p_value=RCC_MCO2Div_2;
  if(prescaler==MCO_DIV3) p_value=RCC_MCO2Div_3;
  if(prescaler==MCO_DIV4) p_value=RCC_MCO2Div_4;
  if(prescaler==MCO_DIV5) p_value=RCC_MCO2Div_5;

  // Clock an PC9 ausgeben
  if(source==MCO2_SYSCLOCK) RCC_MCO2Config(RCC_MCO2Source_SYSCLK, p_value);
  if(source==MCO2_PLLI2SCLK) RCC_MCO2Config(RCC_MCO2Source_PLLI2SCLK, p_value);
  if(source==MCO2_HSE) RCC_MCO2Config(RCC_MCO2Source_HSE, p_value);
  if(source==MCO2_PLLCLK) RCC_MCO2Config(RCC_MCO2Source_PLLCLK, p_value);

  GPIO_PinAFConfig(MCO2_GPIO_PORT, MCO2_GPIO_SOURCE, GPIO_AF_MCO);
}
#endif

